/***************************************************************************************************
*****    This file is part of KardsGT.                                                         *****
*****                                                                                          *****
*****    Copyright (C) 2005-2008  John Schneiderman <JohnMS@member.fsf.org>                    *****
*****                                                                                          *****
*****    This program is free software: you can redistribute it and/or modify it under the     *****
*****    terms of the GNU General Public License as published by the Free Software Foundation, *****
*****    either version 3 of the License, or (at your option) any later version.               *****
*****                                                                                          *****
*****    This program is distributed in the hope that it will be useful, but WITHOUT ANY       *****
*****    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A       *****
*****    PARTICULAR PURPOSE.  See the GNU General Public License for more details.             *****
*****                                                                                          *****
*****    You should have received a copy of the GNU General Public License along with this     *****
*****    program. If not, see <http://www.gnu.org/licenses/>.                                  *****
***************************************************************************************************/
#include "cribbageboard.h"
#include "cribbageblock.h"
#include "cribbagepeg.h"
#include "kardsgterror.h"

#include <QPainter>
#include <QPixmap>
#include <QRect>
#include <QBrush>
#include <QString>
#include <QPaintEvent>
#include <QPalette>

CribbageBoard::CribbageBoard(QWidget *parent): QWidget(parent)
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    setBackgroundRole(QPalette::Background);
    setAutoFillBackground (true);
    QPalette pal(this->palette());
    pal.setColor(QPalette::Background, QColor(210, 180, 140)); //A light brown.
    this->setPalette(pal);

    //Create our pegs.
    for (int i=0; i < NUMBER_OF_PEGS; ++i)
    {
        m_redPegs.push_back(new CribbagePeg(Qt::red, QPoint(ZERO_ZONE_X + 1, ZERO_ZONE_Y + (i * 2) + 1), (i == 0) ? true : false)); // i == 0 then it is a lead CribbagePeg
        m_bluePegs.push_back(new CribbagePeg(Qt::blue, QPoint(ZERO_ZONE_X + 4, ZERO_ZONE_Y + (i * 2) + 1), (i == 0) ? true : false)); // i == 0 then it is a lead CribbagePeg
    }
    m_legendPegs.push_back(pair<CribbagePeg *, QString>(new CribbagePeg(Qt::red, QPoint(ZERO_ZONE_X + 1, ZERO_ZONE_Y + 30), true), "Red Player"));
    m_legendPegs.push_back(pair<CribbagePeg *, QString>(new CribbagePeg(Qt::blue, QPoint(ZERO_ZONE_X + 1, ZERO_ZONE_Y + 35), true), "Blue Player"));

    //Create our blocks
    for (int i=0; i < NUMBER_VERTICAL_BLOCKS; ++i)
    {
        m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_PEGS_PER_BLOCK, FIRST_ZONE_X, FIRST_ZONE_Y + (i * 10)));
        m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_PEGS_PER_BLOCK, THIRD_ZONE_X, THIRD_ZONE_Y + (i * 10)));
        m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_PEGS_PER_BLOCK, FIFTH_ZONE_X, FIFTH_ZONE_Y + (i * 10)));
        m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_PEGS_PER_BLOCK, FIRST_ZONE_X + BLOCK_OFFSET, FIRST_ZONE_Y + (i * 10)));
        m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_PEGS_PER_BLOCK, THIRD_ZONE_X - BLOCK_OFFSET, THIRD_ZONE_Y + (i * 10)));
        m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_PEGS_PER_BLOCK, FIFTH_ZONE_X + BLOCK_OFFSET, FIFTH_ZONE_Y + (i * 10)));
    }
    m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_PEGS_PER_BLOCK, SECOND_ZONE_X, SECOND_ZONE_Y, CribbageBlock::HORIZONTAL));
    m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_PEGS_PER_BLOCK, SECOND_ZONE_X + (NUMBER_PEGS_PER_BLOCK * 2), SECOND_ZONE_Y, CribbageBlock::HORIZONTAL));
    m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_PEGS_PER_BLOCK, FOURTH_ZONE_X, FOURTH_ZONE_Y, CribbageBlock::HORIZONTAL));
    m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_OF_PEGS, ZERO_ZONE_X, ZERO_ZONE_Y));
    m_blocks.push_back(new CribbageBlock(Qt::darkRed, NUMBER_OF_STAKES, STAKE_ZONE_X, STAKE_ZONE_Y, CribbageBlock::HORIZONTAL));
    m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_PEGS_PER_BLOCK, SECOND_ZONE_X, SECOND_ZONE_Y + BLOCK_OFFSET, CribbageBlock::HORIZONTAL));
    m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_PEGS_PER_BLOCK, SECOND_ZONE_X + (NUMBER_PEGS_PER_BLOCK * 2), SECOND_ZONE_Y + BLOCK_OFFSET, CribbageBlock::HORIZONTAL));
    m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_PEGS_PER_BLOCK, FOURTH_ZONE_X, FOURTH_ZONE_Y - BLOCK_OFFSET, CribbageBlock::HORIZONTAL));
    m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_OF_PEGS, ZERO_ZONE_X + BLOCK_OFFSET, ZERO_ZONE_Y));
    m_blocks.push_back(new CribbageBlock(Qt::darkBlue, NUMBER_OF_STAKES, STAKE_ZONE_X, STAKE_ZONE_Y + BLOCK_OFFSET, CribbageBlock::HORIZONTAL));

    // This is the first pegging zone 0-36
    m_peggingsZones.push_back(new QPoint(FIRST_ZONE_X, FIRST_ZONE_Y + 69));
    // This is the second pegging zone 36-46
    m_peggingsZones.push_back(new QPoint(SECOND_ZONE_X, SECOND_ZONE_Y + 1));
    // This is the third pegging zone 46-81
    m_peggingsZones.push_back(new QPoint(THIRD_ZONE_X + 1, THIRD_ZONE_Y + 1));
    // This is the fourth zone 81-86
    m_peggingsZones.push_back(new QPoint(FOURTH_ZONE_X + 8, FOURTH_ZONE_Y + 1));
    // This is the fifth zone 86-121
    m_peggingsZones.push_back(new QPoint(FIFTH_ZONE_X + 1, FIFTH_ZONE_Y + 69));
    // This is the sixth zone (Stakes won) 121
    m_peggingsZones.push_back(new QPoint(SIXTH_ZONE_X + 1, SIXTH_ZONE_Y + 1));
    // This is the stake zone
    m_peggingsZones.push_back(new QPoint(STAKE_ZONE_X, STAKE_ZONE_Y + 1));
}

CribbageBoard::~CribbageBoard()
{
    int size=static_cast<int>(m_redPegs.size());

    for (int i=0; i < size; ++i)
        delete m_redPegs[i];
    size=static_cast<int>(m_bluePegs.size());
    for (int i=0; i < size; ++i)
        delete m_bluePegs[i];
    size=static_cast<int>(m_peggingsZones.size());
    for (int i=0; i < size; ++i)
        delete m_peggingsZones[i];
    size=static_cast<int>(m_blocks.size());
    for (int i=0; i < size; ++i)
        delete m_blocks[i];
    size=static_cast<int>(m_legendPegs.size());
    for (int i=0; i < size; ++i)
        delete m_legendPegs[i].first;
}

pair<int, int> CribbageBoard::scores(const QColor &pegColour) const
{
    pair<int, int> playerScores;

    if (pegColour == Qt::red)
    {
        if (m_redPegs.front()->isFront())
        {
            playerScores.first = m_redPegs[0]->score();
            playerScores.second = m_redPegs[1]->score();
        }
        else
        {
            playerScores.first = m_redPegs[1]->score();
            playerScores.second = m_redPegs[0]->score();
        }
    }
    else if (pegColour == Qt::blue)
    {
        if (m_bluePegs.front()->isFront())
        {
            playerScores.first = m_bluePegs[0]->score();
            playerScores.second = m_bluePegs[1]->score();
        }
        else
        {
            playerScores.first = m_bluePegs[1]->score();
            playerScores.second = m_bluePegs[0]->score();
        }
    }
    else
        throw KardsGTError("CribbageBoard", "scores", "Failed to determine colour of the peg!");
    return playerScores;
}

void CribbageBoard::setScores(const QColor &pegColour, pair<int, int> scores)
{
    // See if we're resetting the scores
    if (scores.second != 0)
    {
        pegScore(pegColour, scores.second);
        pegScore(pegColour, scores.first);
    }
    else
    {
        pegScore(pegColour, scores.first);
        pegScore(pegColour, scores.second);
    }
}

void CribbageBoard::pegScore(const QColor &pegColour, int score)
{
    int leadPeg=0, nonLeadPeg=0;

    if ((pegColour == m_redPegs.front()->colour()))
    {
        // Find leading CribbagePeg
        if (m_redPegs.front()->isFront())
        {
            leadPeg=0;
            nonLeadPeg=1;
        }
        else
        {
            leadPeg=1;
            nonLeadPeg=0;
        }

        // Return if the score hasn't changed.
        if (score == m_redPegs[leadPeg]->score())
            return;

        // If we're starting out, move the first CribbagePeg to the leading position
        if (m_redPegs[leadPeg]->score() == 0)
        {
            movePeg(*m_redPegs[leadPeg], score);
            m_redPegs[leadPeg]->setScore(score);
        }
        // Move the non leading CribbagePeg to the leading new score position.
        else if (m_redPegs[leadPeg]->score() < SIXTH_ZONE_ENDING)
        {
            movePeg(*m_redPegs[nonLeadPeg], score);
            m_redPegs[nonLeadPeg]->setScore(score);

            // Reset who's in the front
            m_redPegs[leadPeg]->setLead(false);
            m_redPegs[nonLeadPeg]->setLead(true);
        }
        else
        {
            movePeg(*m_redPegs[nonLeadPeg], FIFTH_ZONE_ENDING);
            m_redPegs[nonLeadPeg]->setScore(FIFTH_ZONE_ENDING);

            // Reset who's in the front
            m_redPegs[leadPeg]->setLead(false);
            m_redPegs[nonLeadPeg]->setLead(true);
        }

    }
    else if (pegColour == m_bluePegs.front()->colour())
    {
        // Find leading CribbagePeg
        if (m_bluePegs.front()->isFront())
        {
            leadPeg=0;
            nonLeadPeg=1;
        }
        else
        {
            leadPeg=1;
            nonLeadPeg=0;
        }

        // Return if the score hasn't changed.
        if (score == m_bluePegs[leadPeg]->score())
            return;

        // If we're starting out, move the first CribbagePeg to the leading position
        if (m_bluePegs[leadPeg]->score() == 0)
        {
            movePeg(*m_bluePegs[leadPeg], score);
            m_bluePegs[leadPeg]->setScore(score);
        }
        // Move the non leading CribbagePeg to the leading new score position.
        else if (m_bluePegs[leadPeg]->score() < SIXTH_ZONE_ENDING)
        {
            movePeg(*m_bluePegs[nonLeadPeg], score);
            m_bluePegs[nonLeadPeg]->setScore(score);

            // Reset who's in the front
            m_bluePegs[leadPeg]->setLead(false);
            m_bluePegs[nonLeadPeg]->setLead(true);
        }
        else
        {
            movePeg(*m_bluePegs[nonLeadPeg], FIFTH_ZONE_ENDING);
            m_bluePegs[nonLeadPeg]->setScore(FIFTH_ZONE_ENDING);

            // Reset who's in the front
            m_bluePegs[leadPeg]->setLead(false);
            m_bluePegs[nonLeadPeg]->setLead(true);
        }
    }
    else
        throw KardsGTError("CribbageBoard", "pegScore", "Failed to determine colour of the peg!");
}

void CribbageBoard::movePeg(CribbagePeg &peg, int newScore)
{
    // Are we in the starting position
    if (newScore < ZERO_ZONE_ENDING)
        peg.setPosition(peg.startPosition());
    // Are we in the first zone
    else if ((newScore >= ZERO_ZONE_ENDING) && (newScore < FIRST_ZONE_ENDING))
        peg.setPosition(peg.x(), m_peggingsZones[0]->y() - (newScore * 2) + 2);
    // Are we in the second zone
    else if ((newScore >= FIRST_ZONE_ENDING) && (newScore < SECOND_ZONE_ENDING))
        if (peg.colour() == Qt::red)
            peg.setPosition(m_peggingsZones[1]->x() + ((newScore - FIRST_ZONE_ENDING) * 2), m_peggingsZones[1]->y());
        else
            peg.setPosition(m_peggingsZones[1]->x() + ((newScore - FIRST_ZONE_ENDING) * 2), m_peggingsZones[1]->y() + 3);
    // Are we in the third zone
    else if ((newScore >= SECOND_ZONE_ENDING) && (newScore < THIRD_ZONE_ENDING))
        if (peg.colour() == Qt::red)
            peg.setPosition(m_peggingsZones[2]->x(), m_peggingsZones[2]->y() + ((newScore - 46) * 2));
        else
            peg.setPosition(m_peggingsZones[2]->x() - 3, m_peggingsZones[2]->y() + ((newScore - 46) * 2));
    // Are we in the fourth zone
    else if ((newScore >= THIRD_ZONE_ENDING) && (newScore < FOURTH_ZONE_ENDING))
        if (peg.colour() == Qt::red)
            peg.setPosition(m_peggingsZones[3]->x() - ((newScore - 81) * 2), m_peggingsZones[3]->y());
        else
            peg.setPosition(m_peggingsZones[3]->x() - ((newScore - 81) * 2), m_peggingsZones[3]->y() - 3);
    // Are we in the fifth zone
    else if ((newScore >= FOURTH_ZONE_ENDING) && (newScore < FIFTH_ZONE_ENDING))
        if (peg.colour() == Qt::red)
            peg.setPosition(m_peggingsZones[4]->x(), m_peggingsZones[4]->y() - ((newScore - 86) * 2));
        else
            peg.setPosition(m_peggingsZones[4]->x() + 3, m_peggingsZones[4]->y() - ((newScore - 86) * 2));
    // Are we in the sixth zone
    else if (newScore >= FIFTH_ZONE_ENDING)
        peg.setPosition(SIXTH_ZONE_X, SIXTH_ZONE_Y);
    else
        throw KardsGTError("CribbageBoard", "movePeg", "Failed to determine the zone to place the peg in.");
    peg.setScore(newScore);
    update();
    updateGeometry();
}

void CribbageBoard::pegStakes(const QColor &pegColour, int stakes)
{
    if (stakes > NUMBER_OF_STAKES)
        return;

    if ((pegColour == m_redPegs.front()->colour()))
    {
        if (m_redPegs.back()->score() == stakes)
            return;
        else
            m_redPegs.back()->setPosition(m_peggingsZones.back()->x() + ((stakes - 1) * 2), m_peggingsZones.back()->y());
        m_redPegs.back()->setScore(stakes);
    }
    else if ((pegColour == m_bluePegs.front()->colour()))
    {
        if (m_bluePegs.back()->score() == stakes)
            return;
        else
            m_bluePegs.back()->setPosition(m_peggingsZones.back()->x() + ((stakes - 1) * 2), m_peggingsZones.back()->y() + 3);
        m_bluePegs.back()->setScore(stakes);
    }
    else
        throw KardsGTError("CribbageBoard", "pegStakes", "Failed to determine colour of the peg!");
    update();
    updateGeometry();
}

void CribbageBoard::reset()
{
    int size = NUMBER_OF_PEGS - 1; // The stakes peg is assumed to be the last peg.

    for (int i=0; i < size; ++i)
        m_redPegs[i]->reset();
    for (int i=0; i < size; ++i)
        m_bluePegs[i]->reset();
    update();
    updateGeometry();
}

void CribbageBoard::clear()
{
    int size = NUMBER_OF_PEGS;

    for (int i=0; i < size; ++i)
        m_redPegs[i]->reset();
    for (int i=0; i < size; ++i)
        m_bluePegs[i]->reset();
    update();
    updateGeometry();
}

void CribbageBoard::setName(const QColor &pegColour, const QString &name)
{
    int size=m_legendPegs.size();

    for (int i=0; i < size; ++i)
        if (m_legendPegs[i].first->colour() == pegColour)
        {
            m_legendPegs[i].second =  name;
            break;
        }
    update();
    updateGeometry();
}

QSize CribbageBoard::sizeHint() const
{
    return QSize(PREFERRED_WIDTH, PREFERRED_HEIGHT);
}

void CribbageBoard::draw(QPainter *painter)
{
    const int TEXT_OFFSET=4; // The space between the blocks and the text

    // Draw the blocks
    for (int i=0; i < static_cast<int>(m_blocks.size()); ++i)
        m_blocks[i]->draw(painter);

    // Draw the lines connecting each CribbageBlock
    painter->setPen(QPen(Qt::darkRed));
    painter->drawLine(FIRST_ZONE_X, FIRST_ZONE_Y, SECOND_ZONE_X, SECOND_ZONE_Y);
    painter->drawLine(SECOND_ZONE_X + (NUMBER_PEGS_PER_BLOCK * 4), SECOND_ZONE_Y, THIRD_ZONE_X, THIRD_ZONE_Y);
    painter->drawLine(THIRD_ZONE_X, THIRD_ZONE_Y + (NUMBER_PEGS_PER_BLOCK * NUMBER_VERTICAL_BLOCKS * 2), FOURTH_ZONE_X + (NUMBER_PEGS_PER_BLOCK * 2), FOURTH_ZONE_Y);
    painter->drawLine(FOURTH_ZONE_X, FOURTH_ZONE_Y, FIFTH_ZONE_X, FIFTH_ZONE_Y + (NUMBER_PEGS_PER_BLOCK * NUMBER_VERTICAL_BLOCKS * 2));
    painter->setPen(QPen(Qt::darkBlue));
    painter->drawLine(FIRST_ZONE_X + BLOCK_OFFSET, FIRST_ZONE_Y, SECOND_ZONE_X, SECOND_ZONE_Y + BLOCK_OFFSET);
    painter->drawLine(SECOND_ZONE_X + (NUMBER_PEGS_PER_BLOCK * 4), SECOND_ZONE_Y + BLOCK_OFFSET, THIRD_ZONE_X - BLOCK_OFFSET, THIRD_ZONE_Y);
    painter->drawLine(THIRD_ZONE_X - BLOCK_OFFSET, THIRD_ZONE_Y  + (NUMBER_PEGS_PER_BLOCK * NUMBER_VERTICAL_BLOCKS * 2), FOURTH_ZONE_X + (NUMBER_PEGS_PER_BLOCK * 2), FOURTH_ZONE_Y - BLOCK_OFFSET);
    painter->drawLine(FOURTH_ZONE_X, FOURTH_ZONE_Y - BLOCK_OFFSET, FIFTH_ZONE_X + BLOCK_OFFSET, FIFTH_ZONE_Y + (NUMBER_PEGS_PER_BLOCK * NUMBER_VERTICAL_BLOCKS * 2));
    painter->setPen(QPen(Qt::black));

    //Draw the numbers and labels
    painter->setFont(QFont("Courier", 4));
    painter->setBrush(QBrush(Qt::black));
    for (int i=0; i < NUMBER_VERTICAL_BLOCKS; ++i)
    {
        painter->drawText(FIRST_ZONE_X + BLOCK_OFFSET + TEXT_OFFSET, FIRST_ZONE_Y + (i * 10), QString("%1").arg(40 - ((i + 1) * NUMBER_PEGS_PER_BLOCK)));
        painter->drawText(THIRD_ZONE_X + BLOCK_OFFSET + 1, THIRD_ZONE_Y + (i * 10) + 10, QString("%1").arg(45 + ((i + 1) * NUMBER_PEGS_PER_BLOCK)));
        painter->drawText(FIFTH_ZONE_X + BLOCK_OFFSET + TEXT_OFFSET, FIFTH_ZONE_Y + (i * 10), QString("%1").arg(125 - ((i + 1) * NUMBER_PEGS_PER_BLOCK)));
    }
    painter->drawText(SECOND_ZONE_X + NUMBER_PEGS_PER_BLOCK + TEXT_OFFSET, SECOND_ZONE_Y + (BLOCK_OFFSET * 4), QString("%1").arg(40));
    painter->drawText(SECOND_ZONE_X + (NUMBER_PEGS_PER_BLOCK * 2) + (TEXT_OFFSET * 2), SECOND_ZONE_Y + (BLOCK_OFFSET * 4), QString("%1").arg(45));
    painter->drawText(FOURTH_ZONE_X - TEXT_OFFSET, FOURTH_ZONE_Y - BLOCK_OFFSET, QString("%1").arg(85));
    painter->drawEllipse(SIXTH_ZONE_X, SIXTH_ZONE_Y, 2, 2); // The finish CribbagePeg.
    painter->drawText(SIXTH_ZONE_X - (TEXT_OFFSET * 1), SIXTH_ZONE_Y - TEXT_OFFSET + 3, "Win");
    painter->drawText(STAKE_ZONE_X - (TEXT_OFFSET * 5), STAKE_ZONE_Y + TEXT_OFFSET + 1, "Stakes");
    painter->drawText(SECOND_ZONE_X - TEXT_OFFSET, SECOND_ZONE_Y - (TEXT_OFFSET * 2), "Cribbage");
    painter->drawText(SECOND_ZONE_X, SECOND_ZONE_Y - (TEXT_OFFSET), "Board");

    // Draw each players pegs
    for (int i=0; i < NUMBER_OF_PEGS; ++i)
    {
        m_redPegs[i]->draw(painter);
        m_bluePegs[i]->draw(painter);
    }
    for (int i=0; i < static_cast<int>(m_legendPegs.size()); ++i)
    {
        m_legendPegs[i].first->draw(painter);
        painter->drawText(m_legendPegs[i].first->x() + 2, m_legendPegs[i].first->y() + 1, m_legendPegs[i].second);
    }
}

void CribbageBoard::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    int side = qMin(width(), height());

    painter.setViewport((width() - side) / 2 - event->rect().x(), (height() - side) / 2 - event->rect().y(), side, side);
    painter.setWindow(0, 0, 50, 80);
    draw(&painter);
}
